/// <reference types="frida-gum" />
import { NonNullNativeStruct } from "../../utils/native-struct";
/** Represents a `Il2CppClass`. */
declare class Il2CppClass extends NonNullNativeStruct {
    /** Gets the actual size of the instance of the current class. */
    get actualInstanceSize(): number;
    /** Gets the array class which encompass the current class. */
    get arrayClass(): Il2Cpp.Class;
    /** Gets the size of the object encompassed by the current array class. */
    get arrayElementSize(): number;
    /** Gets the name of the assembly in which the current class is defined. */
    get assemblyName(): string;
    /** Gets the class that declares the current nested class. */
    get declaringClass(): Il2Cpp.Class | null;
    /** Gets the encompassed type of this array, reference, pointer or enum type. */
    get baseType(): Il2Cpp.Type | null;
    /** Gets the class of the object encompassed or referred to by the current array, pointer or reference class. */
    get elementClass(): Il2Cpp.Class | null;
    /** Gets the fields of the current class. */
    get fields(): Il2Cpp.Field[];
    /** Gets the flags of the current class. */
    get flags(): number;
    /** Gets the amount of generic parameters of this generic class. */
    get genericParameterCount(): number;
    /** Determines whether the GC has tracking references to the current class instances. */
    get hasReferences(): boolean;
    /** Determines whether ther current class has a valid static constructor. */
    get hasStaticConstructor(): boolean;
    /** Gets the image in which the current class is defined. */
    get image(): Il2Cpp.Image;
    /** Gets the size of the instance of the current class. */
    get instanceSize(): number;
    /** Determines whether the current class is abstract. */
    get isAbstract(): boolean;
    /** Determines whether the current class is blittable. */
    get isBlittable(): boolean;
    /** Determines whether the current class is an enumeration. */
    get isEnum(): boolean;
    /** Determines whether the current class is a generic one. */
    get isGeneric(): boolean;
    /** Determines whether the current class is inflated. */
    get isInflated(): boolean;
    /** Determines whether the current class is an interface. */
    get isInterface(): boolean;
    /** Determines whether the current class is a value type. */
    get isValueType(): boolean;
    /** Gets the interfaces implemented or inherited by the current class. */
    get interfaces(): Il2Cpp.Class[];
    /** Gets the methods implemented by the current class. */
    get methods(): Il2Cpp.Method[];
    /** Gets the name of the current class. */
    get name(): string;
    /** Gets the namespace of the current class. */
    get namespace(): string;
    /** Gets the classes nested inside the current class. */
    get nestedClasses(): Il2Cpp.Class[];
    /** Gets the class from which the current class directly inherits. */
    get parent(): Il2Cpp.Class | null;
    /** Gets the rank (number of dimensions) of the current array class. */
    get rank(): number;
    /** Gets a pointer to the static fields of the current class. */
    get staticFieldsData(): NativePointer;
    /** Gets the size of the instance - as a value type - of the current class. */
    get valueSize(): number;
    /** Gets the type of the current class. */
    get type(): Il2Cpp.Type;
    /** Allocates a new object of the current class. */
    alloc(): Il2Cpp.Object;
    /** Gets the field identified by the given name. */
    field<T extends Il2Cpp.Field.Type>(name: string): Il2Cpp.Field<T>;
    /** Builds a generic instance of the current generic class. */
    inflate(...classes: Il2Cpp.Class[]): Il2Cpp.Class;
    /** Calls the static constructor of the current class. */
    initialize(): void;
    /** Determines whether an instance of `other` class can be assigned to a variable of the current type. */
    isAssignableFrom(other: Il2Cpp.Class): boolean;
    /** Determines whether the current class derives from `other` class. */
    isSubclassOf(other: Il2Cpp.Class, checkInterfaces: boolean): boolean;
    /** Gets the method identified by the given name and parameter count. */
    method<T extends Il2Cpp.Method.ReturnType>(name: string, parameterCount?: number): Il2Cpp.Method<T>;
    /** Gets the nested class with the given name. */
    nested(name: string): Il2Cpp.Class;
    /** Allocates a new object of the current class and calls its default constructor. */
    new(): Il2Cpp.Object;
    /** Gets the field with the given name. */
    tryField<T extends Il2Cpp.Field.Type>(name: string): Il2Cpp.Field<T> | null;
    /** Gets the method with the given name and parameter count. */
    tryMethod<T extends Il2Cpp.Method.ReturnType>(name: string, parameterCount?: number): Il2Cpp.Method<T> | null;
    /** Gets the nested class with the given name. */
    tryNested(name: string): Il2Cpp.Class | undefined;
    /** */
    toString(): string;
    /** Executes a callback for every defined class. */
    static enumerate(block: (klass: Il2Cpp.Class) => void): void;
}
declare global {
    namespace Il2Cpp {
        class Class extends Il2CppClass {
        }
    }
}
export {};
