/// <reference types="frida-gum" />
/** Garbage collector utility functions. */
declare class Il2CppGC {
    protected constructor();
    /** Gets the heap size in bytes. */
    static get heapSize(): Int64;
    /** Determines whether the garbage collector is disabled. */
    static get isEnabled(): boolean;
    /** Determines whether the garbage collector is incremental. */
    static get isIncremental(): boolean;
    /** Gets the number of nanoseconds the garbage collector can spend in a collection step. */
    static get maxTimeSlice(): Int64;
    /** Gets the used heap size in bytes. */
    static get usedHeapSize(): Int64;
    /** Enables or disables the garbage collector. */
    static set isEnabled(value: boolean);
    /** Sets the number of nanoseconds the garbage collector can spend in a collection step. */
    static set maxTimeSlice(nanoseconds: number | Int64);
    /** Returns the heap allocated objects of the specified class. This variant reads GC descriptors. */
    static choose(klass: Il2Cpp.Class): Il2Cpp.Object[];
    /** Forces a garbage collection of the specified generation. */
    static collect(generation: 0 | 1 | 2): void;
    /** Forces a garbage collection. */
    static collectALittle(): void;
    /** Resumes all the previously stopped threads. */
    static startWorld(): void;
    /** Performs an incremental garbage collection. */
    static startIncrementalCollection(): void;
    /** Stops all threads which may access the garbage collected heap, other than the caller. */
    static stopWorld(): void;
}
declare global {
    namespace Il2Cpp {
        class GC extends Il2CppGC {
        }
    }
}
export {};
