/// <reference types="frida-gum" />
import { NonNullNativeStruct } from "../../utils/native-struct";
/** Represents a `MethodInfo`. */
declare class Il2CppMethod<T extends Il2Cpp.Method.ReturnType> extends NonNullNativeStruct {
    /** Gets the class in which this method is defined. */
    get class(): Il2Cpp.Class;
    /** Gets the flags of the current method. */
    get flags(): number;
    /** Gets the implementation flags of the current method. */
    get implementationFlags(): number;
    /** */
    get fridaSignature(): NativeCallbackArgumentType[];
    /** Gets the amount of generic parameters of this generic method. */
    get genericParameterCount(): number;
    /** Determines whether this method is external. */
    get isExternal(): boolean;
    /** Determines whether this method is generic. */
    get isGeneric(): boolean;
    /** Determines whether this method is inflated (generic with a concrete type parameter). */
    get isInflated(): boolean;
    /** Determines whether this method is static. */
    get isStatic(): boolean;
    /** Determines whether this method is synchronized. */
    get isSynchronized(): boolean;
    /** Gets the access modifier of this method. */
    get modifier(): string;
    /** Gets the name of this method. */
    get name(): string;
    /** Gets the encompassing object of the current method. */
    get object(): Il2Cpp.Object;
    /** Gets the amount of parameters of this method. */
    get parameterCount(): number;
    /** Gets the parameters of this method. */
    get parameters(): Il2Cpp.Parameter[];
    /** Gets the relative virtual address (RVA) of this method. */
    get relativeVirtualAddress(): NativePointer;
    /** Gets the return type of this method. */
    get returnType(): Il2Cpp.Type;
    /** Gets the virtual address (VA) to this method. */
    get virtualAddress(): NativePointer;
    /** Replaces the body of this method. */
    set implementation(block: (this: Il2Cpp.Class | Il2Cpp.Object, ...parameters: any[]) => T);
    /** Creates a generic instance of the current generic method. */
    inflate<R extends Il2Cpp.Method.ReturnType = T>(...classes: Il2Cpp.Class[]): Il2Cpp.Method<R>;
    /** Invokes this method. */
    invoke(...parameters: Il2Cpp.Parameter.Type[]): T;
    /** Gets the overloaded method with the given parameter types. */
    overload(...parameterTypes: string[]): Il2Cpp.Method<T>;
    /** Gets the parameter with the given name. */
    parameter(name: string): Il2Cpp.Parameter;
    /** Restore the original method implementation. */
    revert(): void;
    /** Gets the overloaded method with the given parameter types. */
    tryOverload<U extends Il2Cpp.Method.ReturnType = T>(...parameterTypes: string[]): Il2Cpp.Method<U> | undefined;
    /** Gets the parameter with the given name. */
    tryParameter(name: string): Il2Cpp.Parameter | undefined;
    /** */
    toString(): string;
}
declare global {
    namespace Il2Cpp {
        class Method<T extends Il2Cpp.Method.ReturnType = Il2Cpp.Method.ReturnType> extends Il2CppMethod<T> {
        }
        namespace Method {
            type ReturnType = void | Il2Cpp.Field.Type;
            const enum Attributes {
                MemberAccessMask = 7,
                PrivateScope = 0,
                Private = 1,
                FamANDAssem = 2,
                Assembly = 3,
                Family = 4,
                FamORAssem = 5,
                Public = 6,
                Static = 16,
                Final = 32,
                Virtual = 64,
                HideBySig = 128,
                CheckAccessOnOverride = 512,
                VtableLayoutMask = 256,
                ReuseSlot = 0,
                NewSlot = 256,
                Abstract = 1024,
                SpecialName = 2048,
                PinvokeImpl = 8192,
                UnmanagedExport = 8,
                RTSpecialName = 4096,
                ReservedMask = 53248,
                HasSecurity = 16384,
                RequireSecObject = 32768
            }
            const enum ImplementationAttribute {
                CodeTypeMask = 3,
                IntermediateLanguage = 0,
                Native = 1,
                OptimizedIntermediateLanguage = 2,
                Runtime = 3,
                ManagedMask = 4,
                Unmanaged = 4,
                Managed = 0,
                ForwardRef = 16,
                PreserveSig = 128,
                InternalCall = 4096,
                Synchronized = 32,
                NoInlining = 8,
                AggressiveInlining = 256,
                NoOptimization = 64,
                SecurityMitigations = 1024,
                MaxMethodImplVal = 65535
            }
        }
    }
}
export {};
